/*! TODO: Application description here

This file provided by [rust-cli-boilerplate](https://github.com/ssokolow/rust-cli-boilerplate)
*/
// Copyright 2017-2021, Stephan Sokolow

// Make rustc's built-in lints more strict and set clippy into a whitelist-based configuration so
// we see new lints as they get written, then opt out of ones we have seen and don't want
#![warn(warnings, rust_2018_idioms)]
#![warn(clippy::all, clippy::pedantic, clippy::restriction)]
#![allow(clippy::float_arithmetic, clippy::implicit_return, clippy::needless_return)]
#![forbid(unsafe_code)] // Enforce my policy of only allowing it in my own code as a last resort

#![allow(warnings)]
#![allow(unused_imports)]

// stdlib imports
//use std::{any, convert::TryInto};
use std::{convert::TryInto};
use std::io;

// 3rd-party imports
use anyhow::{Context, Result};
use structopt::{clap, StructOpt};

#[macro_use]
extern crate serde_derive;

use tkdr::crypt_lib::{encrypt, decrypt};
use tkdr::tera_lib::{hash_from_data, data_templated, hash_content};
use tkdr::randomkey::{RandomKey};
use key_of_life::get_key;

// use log::{debug, error, info, trace, warn};
// use actix_web::{guard, middleware, web, App, HttpServer};
use dotenv::dotenv;
use tokio::fs::File;
use tokio_util::codec::{BytesCodec, FramedRead};
use hyper::service::{make_service_fn, service_fn};
use hyper::{Body, Method, Request, Response, Server, StatusCode};

use crate::defs::{BxDynResult};

/// The verbosity level when no `-q` or `-v` arguments are given, with `0` being `-q`
pub const DEFAULT_VERBOSITY: u64 = 1;

const PKG_VERSION: &'static str = env!("CARGO_PKG_VERSION");
// const PKG_VERSION: Option<&'static str> = option_env!("CARGO_PKG_VERSION");
const PKG_NAME: &'static str = env!("CARGO_PKG_NAME");
const PKG_AUTHORS: &'static str = env!("CARGO_PKG_AUTHORS");

/// Default KEY PATH 
pub const KEY_PATH: &str = ".k";
// Local imports
mod app;
mod helpers;
mod cmds;
// mod dsc;
mod defs;
mod utils;
mod errors;
mod clouds;
mod web;

/// cmd main
// async fn cmd_main() ->  Result<(), Box<dyn std::error::Error>> { 
async fn cmd_main() -> BxDynResult<String> { 
  // Parse command-line arguments (exiting on parse error, --version, or --help)
  let opts = app::CliOpts::from_args();

  // Configure logging output so that -q is "decrease verbosity" rather than instant silence
  let verbosity = opts
      .boilerplate
      .verbose
      .saturating_add(app::DEFAULT_VERBOSITY)
      .saturating_sub(opts.boilerplate.quiet);

  stderrlog::new()
      .module(module_path!())
      .quiet(verbosity == 0)
      .verbosity(verbosity.saturating_sub(1).try_into().context("Verbosity too high")?)
      .timestamp(opts.boilerplate.timestamp.unwrap_or(stderrlog::Timestamp::Off))
      .init()
      .context("Failed to initialize logging output")?;
  // If requested, generate shell completions and then exit with status of "success"
  opts.boilerplate.dump_completions.map_or(app::main(opts).await, |shell| {
    app::CliOpts::clap().gen_completions_to(
        app::CliOpts::clap().get_bin_name().unwrap_or_else(|| clap::crate_name!()),
        shell,
        &mut io::stdout(),
    );
    Ok(String::from(""))
  })
  // if let Some(shell) = opts.boilerplate.dump_completions {
  //     app::CliOpts::clap().gen_completions_to(
  //         app::CliOpts::clap().get_bin_name().unwrap_or_else(|| clap::crate_name!()),
  //         shell,
  //         &mut io::stdout(),
  //     );
  //     Ok(())
  // } else {
  //     // Run the actual `main` and rely on `impl Termination` to provide a simple, concise way to
  //     // allow terminal errors that can be changed later as needed but starts out analogous to
  //     // letting an unhandled exception bubble up in something like Python.
  //     // TODO: Experiment with this and look for ways to polish it up further
  //     app::main(opts)
  // }
}
/// Boilerplate to parse command-line arguments, set up logging, and handle bubbled-up `Error`s.
///
/// See `app::main` for the application-specific logic.
// #[actix_rt::main]
// async fn main() -> std::io::Result<()> {  //   Result<()> {
#[tokio::main]
async fn main() -> std::io::Result<()> {  //   Result<()> {
  // // println!("|{}|",&key);
  // let key_path = &envmnt::get_or("KEY_PATH", KEY_PATH);
  // let key = get_key(&key_path,None).await;
  // // let key = get_key(KEY_PATH,None).await;
  // if key.is_empty() {
  //   std::process::exit(0x0100);
  // }
//   cmd_main().await;
  
	let args: Vec<String> = std::env::args().collect();
	// println!("I got {:?} arguments: {:?}.", args.len() - 1, &args[1..]);
	if args.len() > 1 &&  args[1] == "-v"   {
		println!("{} version: {}", PKG_NAME,PKG_VERSION);
		println!("{} authors: {}", PKG_NAME,PKG_AUTHORS);
		return Ok(());
	}
  let web_mode = &envmnt::get_or("WEB_MODE", "");
  if web_mode == ""  {
    cmd_main().await;
  } else {
    let opts = app::CliOpts::from_args();
    // dbg!(opts); 
    match app::main(opts).await {
      Ok(res) => println!("{}",res),
      Err(e) => println!("Errror: {}",e),
    }
    pretty_env_logger::init();

    let addr = "127.0.0.1:1337".parse().unwrap();

    let make_service =
        make_service_fn(|_| async { Ok::<_, hyper::Error>(service_fn(web::response_examples)) });

    let server = Server::bind(&addr).serve(make_service);

    println!("Listening on http://{}", addr);

    if let Err(e) = server.await {
        eprintln!("server error: {}", e);
    }
  }

  Ok(())
}
// vim: set sw=4 sts=4 expandtab :